"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var OBDConnection_1 = require("./OBDConnection");
/**
 * The wrapper of Bluetooth Serial plugin for Cordova.
 */
var BluetoothOBDConnection = /** @class */ (function (_super) {
    __extends(BluetoothOBDConnection, _super);
    function BluetoothOBDConnection() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Open connection to OBD Reader.
     * @returns the observable of opening connection.
     */
    BluetoothOBDConnection.prototype.open = function (config) {
        var _this = this;
        var subject = new rxjs_1.Subject();
        var device = config.device;
        try {
            bluetoothSerial.connect(device.id, function () {
                _this.data$ = new rxjs_1.Subject();
                bluetoothSerial.subscribe('\r>', function (data) {
                    _this.data$.next(data);
                }, function (error) {
                    _this.close();
                    subject.error(error);
                });
                subject.complete();
            }, function (error) { return subject.error(error); });
        }
        catch (e) {
            subject.error(e);
        }
        return subject.asObservable();
    };
    /**
     * Close and dispose resources of OBD Reader connection.
     * @returns the observable of close operation.
     * The events emitted are ignored, wait for complete (successful) or error (failure).
     */
    BluetoothOBDConnection.prototype.close = function () {
        if (this.data$)
            this.data$.complete();
        bluetoothSerial.unsubscribe();
        bluetoothSerial.disconnect();
        return rxjs_1.EMPTY;
    };
    /**
     * Listener for data received from OBD Reader.
     * @returns Observable of data received from OBD Reader.
     */
    BluetoothOBDConnection.prototype.onData = function () {
        return this.data$.asObservable();
    };
    /**
     * Send a command to OBD Reader.
     * @returns the observable of send operation.
     * The events emitted are ignored, wait for complete (successful) or error (failure).
     */
    BluetoothOBDConnection.prototype.send = function (command) {
        var subject = new rxjs_1.Subject();
        bluetoothSerial.write(command, function () { return subject.complete(); }, function (error) { return subject.error(error); });
        return subject.asObservable();
    };
    /**
     * Marshall the string data into bytes array.
     * @param data the string data.
     * @returns the bytes array.
     */
    BluetoothOBDConnection.marshall = function (data) {
        if (!data) {
            return new Uint8Array(0);
        }
        var bytes = new Uint8Array(data.length);
        for (var i = 0; i < bytes.length; i++) {
            bytes[i] = data.charCodeAt(i);
        }
        return bytes;
    };
    /**
     * Unmarshall the bytes array into data string.
     * @param bytes the bytes array.
     * @returns the string data.
     */
    BluetoothOBDConnection.unmarshall = function (bytes) {
        return String.fromCharCode.apply(null, bytes);
    };
    return BluetoothOBDConnection;
}(OBDConnection_1.OBDConnection));
exports.BluetoothOBDConnection = BluetoothOBDConnection;
//# sourceMappingURL=BluetoothOBDConnection.js.map