"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var net = require("net");
var rxjs_1 = require("rxjs");
/**
 * The wrapper net.Socket connection.
 */
var NodeOBDConnection = /** @class */ (function () {
    function NodeOBDConnection() {
    }
    /**
     * Open connection to OBD Reader.
     * @returns the observable of opening connection.
     */
    NodeOBDConnection.prototype.open = function (config) {
        var _this = this;
        var port = config.port, host = config.host;
        var subject = new rxjs_1.Subject();
        this.socket = net.connect(port, host, function () {
            _this.data$ = new rxjs_1.Subject();
            _this.socket.on('data', function (input) { return _this.data$.next(input.toString('utf8')); });
            subject.complete();
        });
        this.socket.on('error', function (error) { return subject.error(error); });
        return subject.asObservable();
    };
    /**
     * Send a command to OBD Reader.
     * @returns the observable of send operation.
     * The events emitted are ignored, wait for complete (successful) or error (failure).
     */
    NodeOBDConnection.prototype.send = function (command) {
        this.socket.write(command);
        return rxjs_1.EMPTY;
    };
    /**
     * Listener for data received from OBD Reader.
     * @returns Observable of data received from OBD Reader.
     */
    NodeOBDConnection.prototype.onData = function () {
        return this.data$.asObservable();
    };
    /**
     * Close and dispose resources of OBD Reader connection.
     * @returns the observable of close operation.
     * The events emitted are ignored, wait for complete (successful) or error (failure).
     */
    NodeOBDConnection.prototype.close = function () {
        this.data$.complete();
        this.socket.end();
        this.socket.destroy();
        return rxjs_1.EMPTY;
    };
    return NodeOBDConnection;
}());
exports.NodeOBDConnection = NodeOBDConnection;
//# sourceMappingURL=NodeOBDConnection.js.map