"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var OBD_OUTPUT_MESSAGE_TYPES;
(function (OBD_OUTPUT_MESSAGE_TYPES) {
    OBD_OUTPUT_MESSAGE_TYPES["MODE_01"] = "41";
    OBD_OUTPUT_MESSAGE_TYPES["MODE_09"] = "09";
})(OBD_OUTPUT_MESSAGE_TYPES || (OBD_OUTPUT_MESSAGE_TYPES = {}));
var OBD_NO_DATA = 'NO DATA';
var OBD_PROMPT = '>';
function obdReader() {
    return function (source) {
        return source.lift(new OBDReaderOperator());
    };
}
exports.obdReader = obdReader;
var OBDReaderOperator = /** @class */ (function () {
    function OBDReaderOperator() {
    }
    OBDReaderOperator.prototype.call = function (subscriber, source) {
        return source.subscribe(new OBDReaderSubscriber(subscriber));
    };
    return OBDReaderOperator;
}());
var OBDReaderSubscriber = /** @class */ (function (_super) {
    __extends(OBDReaderSubscriber, _super);
    function OBDReaderSubscriber(destination) {
        var _this = _super.call(this, destination) || this;
        /**
         * Internal buffer of bytes received from OBD.
         */
        _this.memento = [];
        /**
         * Internal buffer of the entire output for debug propose.
         */
        _this.buffer = '';
        return _this;
    }
    OBDReaderSubscriber.prototype._next = function (data) {
        this.buffer += data;
        if (OBDReaderSubscriber.hasPrompt(data)) {
            var bytes = this.memento;
            if (OBDReaderSubscriber.isOutput(bytes, OBD_OUTPUT_MESSAGE_TYPES.MODE_01)) {
                this.destination.next(bytes);
            }
            else if (bytes && bytes.length && bytes[0] === OBD_NO_DATA) {
                this.destination.next(bytes);
            }
            else if (this.buffer.indexOf(OBD_OUTPUT_MESSAGE_TYPES.MODE_09) === 0) {
                this.destination.next([this.buffer]);
            }
            else {
                this.destination.error("Prompt received without data. Last call " + JSON.stringify(data) + " and the entire output was " + this.buffer);
            }
            //clear buffers
            this.memento = [];
            this.buffer = '';
        }
        else if (data) {
            if (OBD_NO_DATA === data) {
                this.memento = [data];
            }
            else {
                var bytes = OBDReaderSubscriber.getByteGroupings(data);
                if (OBDReaderSubscriber.isOutput(bytes, OBD_OUTPUT_MESSAGE_TYPES.MODE_01)) {
                    this.memento = bytes;
                }
            }
        }
    };
    /**
     * Determines if the passed buffer/string has a prompt.
     * that indicates it has been completed.
     * @param   data the returned data.
     * @return  flag indicating if the data has a prompt.
     */
    OBDReaderSubscriber.hasPrompt = function (data) {
        // Basically, we check that the a newline has started
        return data.indexOf(OBD_PROMPT) !== -1;
    };
    /**
     * Determines if the passed buffer is an output of desired mode.
     * @param bytes the buffer returned by OBD
     * @param mode the desired mode.
     * @returns Flag indicating that is a valid result.
     */
    OBDReaderSubscriber.isOutput = function (bytes, mode) {
        return bytes && bytes.length && bytes[0] === mode;
    };
    /**
     * Convert the returned bytes into their pairs if possible, or return null.
     * @param  str the returned bytes.
     * @return the returned bytes into their pairs if possible, or return null.
     */
    OBDReaderSubscriber.getByteGroupings = function (str) {
        // Remove white space (if any exists) and get byte groups as pairs
        return str.replace(/ /g, '').match(/.{1,2}/g);
    };
    return OBDReaderSubscriber;
}(rxjs_1.Subscriber));
//# sourceMappingURL=obdReader.js.map